/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   BSD - specific code for x86 processors.
 *   this file is a part of xtrace project
 *
 * $Id: bsd-i386.c,v 1.3 2002/04/23 07:56:40 drozd Exp $
 */

#include <machine/reg.h>
#include <sys/syscall.h>

#include <stdio.h>

#include "xtrace.h"
#include "bsd.h"


static vaddr_t
read_dword(int fd, vaddr_t va)
{
	vaddr_t ret = 0U;

	(void) lseek(fd, va, SEEK_SET);
	(void) read(fd, &ret, sizeof(ret));

	return (ret);
	
}


/*
 * Get call data by stack pointer
 */
int
xtr_get_call_data(struct opt *op, struct calldata *cdp)
{
	static vaddr_t args[MAXARGS], *argsp[MAXARGS];
	struct bsddata *dp = (struct bsddata *) op->data;
	struct reg regs;
	vaddr_t esp;
	int i, scno;

	for (i = 0; i < sizeof(argsp)/sizeof(argsp[0]); i++)
		argsp[i] = args + i;

	if (dp->regsfd < 0)
		return (-1);

	(void) lseek(dp->regsfd, 0L, SEEK_SET);

	if (read(dp->regsfd, &regs, sizeof(regs)) != sizeof(regs)) {
		perror("xtr_get_call_data: read(/proc/pid/regs)");
		return (-1);
	}

	/* clear flags */
	op->flags &= ~XTR_INFUNC;
	op->flags &= ~XTR_INSYSEXIT;
	op->flags &= ~XTR_INSYSENTER;
	
	esp = regs.r_esp + sizeof(int);

	/* find out what syscall if syscall */
	switch (dp->status.why) {
		case S_SCE:
			op->flags |= XTR_INSYSENTER;
			op->scno = regs.r_eax;
			
			/* handle indirect syscalls */
			if (op->scno == SYS___syscall
			    || op->scno == SYS_syscall) {
				
				(void) lseek(dp->procfd, esp, SEEK_SET);
				(void) read(dp->procfd, &scno, sizeof(int));
				
				if (op->scno == SYS_syscall)
					/* int - aligned */
					esp += sizeof(int);
				else
					/* quad - aligned */
					esp += sizeof(quad_t);

				op->scno = scno;
			}

			break;
					
		case S_SCX:
			op->flags |= XTR_INSYSEXIT;
			cdp->retval = regs.r_eax;
			argsp[3] = 0U;
			break;
		default:
			op->flags |= XTR_OTHER;
			argsp[3] = 0U;
			break;
	}
	
	if (cdp->fp == 0U)
		cdp->fp = regs.r_ebp;

	/* read MAXARGS arguments from stack */
	if (dp->status.why == XTR_INSYSENTER) {
		(void) lseek(dp->procfd, esp, SEEK_SET);
		(void) read(dp->procfd, args, sizeof(args));
	} else {
		(void) lseek(dp->procfd, cdp->fp + 8, SEEK_SET);
		(void) read(dp->procfd, args, sizeof(args));
	}

	cdp->argv = argsp;
	cdp->ip = regs.r_eip;
	cdp->ofp = read_dword(dp->procfd, cdp->fp);
	cdp->rta = read_dword(dp->procfd, cdp->fp + 4);
	
	return (0);
}
