/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   ELF manipulation routines needed by xtrace utility, see README for details.
 *   this file is a part of xtrace project
 *
 * $Id: elf.h,v 1.3 2002/02/24 18:25:29 drozd Exp $
 * $Log: elf.h,v $
 * Revision 1.3  2002/02/24 18:25:29  drozd
 * change in interface - svr4 stuff is broken now
 *
 * Revision 1.2  2002/02/21 13:49:56  drozd
 * function trace enabled
 *
 * Revision 1.1.1.1  2002/02/21 08:55:10  drozd
 * import
 *
 */

#ifndef __ELF__H
#define __ELF__H

#include <elf.h>

#include "xtrace.h"

/* open ELF file */
extern Elf32_Ehdr *elf_init(int);

/* close ELF file */
extern void elf_fini(Elf32_Ehdr *);

/* get section header by index */
extern Elf32_Shdr *elf_getshdr(const Elf32_Ehdr *, int);

/* get index of a section by symbolic name */
extern int elf_getindex(const Elf32_Ehdr *, const char *, Elf32_Shdr **);

/* get string from string table by given stringtab index and offset into it */
extern const char *elf_str(const Elf32_Ehdr *, int, int);

/* get load address for specified ELF binary */
extern vaddr_t elf_get_load_addr(const Elf32_Ehdr *);

/* get list of dependent objects for the given ELF object */
char *const *elf_get_depends(const Elf32_Ehdr *ep);

#endif /* __ELF__H */
