/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   Linux, i386 - specific code
 *   this file is a part of xtrace project
 *
 * $Id: linux-unknown.c,v 1.1 2002/02/24 18:25:29 drozd Exp $
 * $Log: linux-unknown.c,v $
 * Revision 1.1  2002/02/24 18:25:29  drozd
 * change in interface - svr4 stuff is broken now
 *
 *
 */


#include <sys/ptrace.h>
#include <linux/user.h>

#include <stdio.h>

#include "xtrace.h"
#include "linux.h"


static struct user_regs_struct *getregs(const struct opt *);
static long read_text_dword(const struct opt *, vaddr_t);

/*
 * Get general purpose registers
 */
struct user_regs_struct *
getregs(const struct opt *op)
{
	static struct user_regs_struct regs;
	
	/* copy the registers */
	if (ptrace(PTRACE_GETREGS, op->pid, 0, &regs) != 0) {
		perror("PTRACE_GETREGS"); return 0;
	}

	return &regs;

}

/*
 * Dump system call information
 * Parameters:
 *    op - ptr to the options
 * Return: void
 */
void
dump_linux_syscall(struct opt *op)
{
	struct user_regs_struct *rp;	/* i386 general - purpose registers */
	int i;
	long prms[6];
	struct linuxdata *dp = (struct linuxdata *) op->data;
	const struct syscall *scp;

	/* copy the registers */
	if ((rp = getregs(op)) == 0)
		return;

	if (dp->flags & LINUX_SYSCALL_RETURN) {

		for (i = dp->ncols; i < 50; i++)
			(void) putchar(' ');

		/* print return value and newline character */
		(void) printf("\t= %lx\n", rp->orig_eax);

		/* toggle flag */
		dp->flags &= ~LINUX_SYSCALL_RETURN;

		/* reset number of columns printed */
		dp->ncols = 0;
	} else {
		/* setup parameters */
		prms[0] = rp->ebx, prms[1] = rp->ecx, prms[2] = rp->edx,
		prms[3] = rp->esi, prms[1] = rp->edi, prms[2] = rp->ebp;

		scp = xtr_lookup_syscall(rp->orig_eax);

		dp->ncols += printf("<%s> (", scp->name);

		if (scp->showparams == 0) {
			/* print parameters ourselves */
			for (i = 0; i < sizeof(prms)/sizeof(prms[0]); i++) {
				dp->ncols += printf("%lx", prms[i]);
				if (i != (sizeof(prms)/sizeof(prms[0]) - 1)) {
					(void) putchar(',');
					++dp->ncols;
				}
			}
		} else {
			/* call callback */
			(*scp->showparams)(op);
		}
		(void) putchar(')');
		++dp->ncols;

		/* toggle flag */
		dp->flags |= LINUX_SYSCALL_RETURN;
	}
	(void) fflush(stdout);
}

/*
 * Read one double word from text segment
 */
long
read_text_dword(const struct opt *op, vaddr_t loc)
{
	long ret = 0;

	ret = ptrace(PTRACE_PEEKTEXT, op->pid, loc, 0);

	return ret;
}

/*
 * Get the stack pointer value
 */
vaddr_t
xtr_get_fp(struct opt *op)
{
	struct user_regs_struct *rp;	/* i386 general - purpose registers */

	/* copy the registers */
	if ((rp = getregs(op)) == 0)
		return 0;

	return rp->ebp;
}


/*
 * Get call data by stack pointer
 */
int
xtr_get_call_data(const struct opt *op, struct calldata *cdp)
{
	struct user_regs_struct *rp;
	static const vaddr_t *argv[128], args[128];

	if (cdp == 0)
		return -1;

	argv[0] = 0;

	if ((rp = getregs(op)) == 0)
		return -1;

	/* fill up call data */
	cdp->ip = rp->eip;

	if (cdp->fp == 0)
		cdp->fp = rp->ebp;

	cdp->ofp = read_text_dword(op, cdp->fp);
	cdp->rta = read_text_dword(op, cdp->fp + 4);
	cdp->argv = argv;

	return 0;	
}
