/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   This file should be #included in $os-syscalls.c
 *   this file is a part of xtrace project
 *
 * $Id: syscall.c,v 1.4 2002/04/23 07:56:40 drozd Exp $
 */

#include <sys/syscall.h>

#include "xtrace.h"

static struct syscall __syscalls[] = {
	{ SYS_syscall,	"syscall", 0,  },
	{ SYS_exit,	"exit", 1, { INT, } },
	{ SYS_fork,	"fork", 0,  },
	{ SYS_read,	"read", 3, { INT, HEX, INT, } },
	{ SYS_write,	"write", 3, { INT, HEX, INT, } },
	{ SYS_open,	"open", 2, { STRING, HEX, } },
	{ SYS_close,	"close", 1, { INT, } },
	{ SYS_wait4,	"wait4", 4, { INT, HEX, INT, HEX, } },
	{ SYS_link,	"link", 2, { STRING, STRING, } },
	{ SYS_unlink,	"unlink", 1, { STRING, } },
	
	{ SYS_chdir,	"chdir", 1, { STRING, } },
	{ SYS_fchdir,	"fchdir", 1, { INT, } },
	{ SYS_mknod,	"mknod", 3, { STRING, INT, INT, } },
	{ SYS_chmod,	"chmod", 2, { STRING, INT, } },
	{ SYS_chown,	"chown", 3, { STRING, INT, INT, } },
	{ SYS_break,	"break", 1, { HEX, } },			/* ??? */
	{ SYS_getfsstat,	"getfsstat", 1, { HEX, } },	/* ??? */
	{ SYS_getpid,	"getpid", 0, },
	{ SYS_mount,	"mount", 4, { STRING, STRING, INT, HEX, } },
	{ SYS_unmount,	"unmount", 2, { STRING, INT, } },
	
	{ SYS_setuid,	"setuid", 1, { INT, } },
	{ SYS_getuid,	"getuid", 0 },
	{ SYS_geteuid,	"geteuid", 0 },
	{ SYS_ptrace,	"ptrace", 4, { INT, INT, HEX, INT} },
	{ SYS_recvmsg,	"recvmsg", 3, { INT, MSGHDR, INT, } },
	{ SYS_sendmsg,	"sendmsg", 3, { INT, MSGHDR, INT, } },
	{ SYS_recvfrom,	"recvfrom", 6, { INT, HEX, INT, INT, SOCKADDR, HEX, } },
	{ SYS_accept,	"accept", 3, { INT, SOCKADDR, HEX, } },
	{ SYS_getpeername,	"getpeername", 3, { INT, SOCKADDR, HEX, } },
	{ SYS_getsockname,	"getsockname", 3, { INT, SOCKADDR, HEX, } },
	{ SYS_access,	"access", 2, { STRING, INT, } },
	
	{ SYS_chflags,	"chflags", 2, { STRING, INT, } },
	{ SYS_fchflags,	"fchflags", 2, { INT, INT, } },
	{ SYS_sync,	"sync", 0 },
	{ SYS_kill,	"kill", 2, { INT, INT, } },
	{ SYS_getppid,	"getppid", 0 },
	{ SYS_dup,	"dup", 1, { INT, } },
	{ SYS_pipe,	"pipe", 1, { HEX, } },
	{ SYS_getegid,	"getegid", 0 },
	{ SYS_profil,	"profil", 4, { STRING, INT, HEX, INT, } },
	{ SYS_ktrace,	"ktrace", 4, { STRING, INT, INT, INT, } },
	{ SYS_getgid,	"getgid", 0 },
	{ SYS_getlogin,	"getlogin", 0 },
	{ SYS_setlogin,	"setlogin", 1, { STRING, } },
	{ SYS_acct,	"acct", 1, { STRING, } },
	{ SYS_sigaltstack,	"sigaltstack", 2, { SIGALTSTACK, SIGALTSTACK} },
	
	{ SYS_ioctl,	"ioctl", 3, { INT, INT, IOCTL, } },
	{ SYS_reboot,	"reboot", 1, { INT, } },
	{ SYS_revoke,	"revoke", 1, { STRING, } },
	{ SYS_symlink,	"symlink", 2, { STRING, STRING, } },
	{ SYS_readlink,	"readlink", 3, { STRING, HEX, INT, } },
	{ SYS_execve,	"execve", 3, { STRING, ARGV, ARGV, } },
	{ SYS_umask,	"umask", 1, { INT, } },
	{ SYS_chroot,	"chroot", 1, { STRING, } },
	{ SYS_msync,	"msync", 3, { HEX, INT, INT, } },
	{ SYS_vfork,	"vfork", 0 },
	{ SYS_sbrk,	"sbrk", 1, { INT, } },
	{ SYS_sstk,	"sstk", 1, { HEX, } },			/* ??? */
	{ SYS_vadvise,	"vadvise", 1, { HEX, } },		/* ??? */
	{ SYS_munmap,	"munmap", 2, { HEX, INT, } },
	{ SYS_mprotect,	"mprotect", 3, { HEX, INT, INT, } },
	{ SYS_madvise,	"madvise", 3, { HEX, INT, INT, } },
	{ SYS_mincore,	"mincore", 3, { HEX, INT, HEX, } },
	{ SYS_getgroups,	"getgroups", 2, { INT, HEX } },
	{ SYS_setgroups,	"setgroups", 2, { INT, HEX, } },
	{ SYS_getpgrp,	"getpgrp", 0 },
	{ SYS_setpgid,	"setpgid", 2, { INT, INT, } },
	{ SYS_setitimer,	"setitimer", 3, { INT, ITIMER, ITIMER, } },
	{ SYS_swapon,	"swapon", 1, { STRING, } },
	{ SYS_getitimer,	"getitimer", 2, { INT, ITIMER, } },
	{ SYS_getdtablesize,	"getdtablesize", 0 },
	{ SYS_dup2,	"dup2", 2, { INT, INT, } },
	{ SYS_fcntl,	"fcntl", 3, { INT, FCNTL, HEX, } },
	
	{ SYS_select,	"select", 5, { INT, HEX, HEX, HEX, TIMEVAL, } },
	{ SYS_fsync,	"fsync", 1, { INT, } },
	{ SYS_setpriority,	"setpriority", 3, { INT, INT, INT, } },
	{ SYS_socket,	"socket", 3, { INT, INT, INT, } },
	{ SYS_connect,	"connect", 3, { INT, SOCKADDR, INT, } },
	{ SYS_getpriority,	"getpriority", 2, { INT, INT, } },
	{ SYS_bind,	"bind", 3, { INT, SOCKADDR, INT, } },
	{ SYS_setsockopt,	"setsockopt", 5, { INT, INT, INT, HEX, INT, } },
	{ SYS_listen,	"listen", 2, { INT, INT, } },
	{ SYS_gettimeofday,	"gettimeofday", 2, { TIMEVAL, TIMEZONE, } },
	{ SYS_getrusage,	"getrusage", 2, { INT, HEX, } },
	{ SYS_getsockopt,	"getsockopt", 5, { INT, INT, INT, HEX, HEX, } },
	{ SYS_readv,	"readv", 3, { INT, IOVEC, INT, } },
	{ SYS_writev,	"writev", 3, { INT, IOVEC, INT, } },
	{ SYS_settimeofday,	"settimeofday", 2, { TIMEVAL, TIMEZONE, } },
	{ SYS_fchown,	"fchown", 3, { INT, INT, INT, } },
	{ SYS_fchmod,	"fchmod", 2, { INT, INT, } },
	{ SYS_setreuid,	"setreuid", 2, { INT, INT, } },
	{ SYS_setregid,	"setregid", 2, { INT, INT, } },
	{ SYS_rename,	"rename", 2, { STRING, STRING, } },
	{ SYS_flock,	"flock", 2, { INT, INT, } },
	{ SYS_mkfifo,	"mkfifo", 2, { STRING, INT, } },
	{ SYS_sendto,	"sendto", 6, { INT, HEX, INT, INT, SOCKADDR, INT, } },
	{ SYS_shutdown,	"shutdown", 2, { INT, INT, } },
	{ SYS_socketpair,	"socketpair", 4, { INT, INT, INT, HEX, } },
	{ SYS_mkdir,	"mkdir", 2, { STRING, INT, } },
	{ SYS_rmdir,	"rmdir", 1, { STRING, } },
	{ SYS_utimes,	"utimes", 2, { STRING, TIMEVAL, } },
	{ SYS_adjtime,	"adjtime", 2, { TIMEVAL, TIMEVAL, } },
	{ SYS_setsid,	"setsid", 0 },
	{ SYS_quotactl,	"quotactl", 4, { STRING, INT, INT, HEX, } },
	{ SYS_nfssvc,	"nfssvc", 2, { INT, HEX, } },	
	{ SYS_statfs,	"statfs", 2, { INT, STATFS, } },
	{ SYS_fstatfs,	"fstatfs", 2, { INT, HEX,  } },
	{ SYS_getfh,	"getfh", 2, { STRING, HEX, } },
	{ SYS_getdomainname,	"getdomainname", 2, { STRING, INT,  } },
	{ SYS_setdomainname,	"setdomainname", 2, { STRING, INT , } },
	{ SYS_uname,	"uname", 0 },
	{ SYS_sysarch,	"sysarch", 2, { INT, HEX, } },
	{ SYS_rtprio,	"rtprio", 3, { INT, INT, RTPRIO, } },
	{ SYS_semsys,	"semsys", 3, { HEX, HEX, HEX, } },	/* ??? */
	{ SYS_msgsys,	"msgsys", 3, { HEX, HEX, HEX, } },	/* ??? */
	{ SYS_shmsys,	"shmsys", 3, { HEX, HEX, HEX, } },	/* ??? */
	{ SYS_pread,	"pread", 4, { INT, HEX, INT, INT, } },
	{ SYS_pwrite,	"pwrite", 4, { INT, HEX, INT, INT, } },
	{ SYS_ntp_adjtime,	"ntp_adjtime", 1, { HEX, } },	/* ??? */
	{ SYS_setgid,	"setgid", 1, { INT, } },
	{ SYS_setegid,	"setegid", 1, { INT, } },
	{ SYS_seteuid,	"seteuid", 1, { INT, } },
	{ SYS_stat,	"stat", 2, { STRING, HEX, } },
	{ SYS_fstat,	"fstat", 2, { INT, HEX, } },
	{ SYS_lstat,	"lstat", 2, { STRING, HEX, } },
	{ SYS_pathconf,	"pathconf", 2, { STRING, INT, } },
	{ SYS_fpathconf,	"fpathconf", 2, { INT, INT, } },
	{ SYS_getrlimit,	"getrlimit", 2, { INT, RLIMIT, } },
	{ SYS_setrlimit,	"setrlimit", 2, { INT, RLIMIT, } },
	{ SYS_getdirentries,	"getdirentries", 4, { INT, HEX, INT, HEX, } },
	{ SYS_mmap,	"mmap", 6, { HEX, INT, INT, INT, INT, INT, } },
	{ SYS___syscall,	"__syscall", 0 },
	{ SYS_lseek,	"lseek", 3, { INT, INT, INT, } },
	{ SYS_truncate,	"truncate", 2, { STRING, INT, } },
	{ SYS_ftruncate,	"ftruncate", 2, { INT, INT, } },
	{ SYS___sysctl,	"__sysctl", 6, { HEX, INT, HEX, HEX, HEX, INT, } },
	{ SYS_mlock,	"mlock", 2, { HEX, INT, } },
	{ SYS_munlock,	"munlock", 2, { HEX, INT, } },
	{ SYS_undelete,	"undelete", 1, { STRING, } },
	{ SYS_futimes,	"futimes", 2, { INT, TIMEVAL, } },
	{ SYS_getpgid,	"getpgid", 1, { INT, } },
	{ SYS_poll,	"poll", 3, { HEX, INT, INT, } },
	{ SYS___semctl,	"__semctl", 4, { INT, INT, INT, HEX, } },
	{ SYS_semget,	"semget", 3, { INT, INT, INT, } },
	{ SYS_semop,	"semop", 3, { INT, HEX, INT, } },
	{ SYS_msgctl,	"msgctl", 3, { INT, INT, HEX, } },
	{ SYS_msgget,	"msgget", 2, { INT, INT, } },
	{ SYS_msgsnd,	"msgsnd", 4, { INT, HEX, INT, INT, } },
	{ SYS_msgrcv,	"msgrcv", 5, { INT, HEX, INT, INT, INT, } },
	{ SYS_shmat,	"shmat", 3, { INT, HEX, INT, } },
	{ SYS_shmctl,	"shmctl", 3, { INT, INT, HEX, } },
	{ SYS_shmdt,	"shmdt", 1, { HEX, } },
	{ SYS_shmget,	"shmget", 3, { INT, INT, INT, } },
	{ SYS_clock_gettime,	"clock_gettime", 2, { INT, TIMESPEC, } },
	{ SYS_clock_settime,	"clock_settime", 2, { INT, TIMESPEC, } },
	{ SYS_clock_getres,	"clock_getres", 2, { INT, TIMESPEC, } },
	{ SYS_nanosleep,	"nanosleep", 2, { TIMESPEC, TIMESPEC, } },
	{ SYS_minherit,	"minherit", 3, { HEX, INT, INT, } },
	{ SYS_rfork,	"rfork", 1, { INT, } },
	{ SYS_openbsd_poll,	"openbsd_poll", 1, { HEX, } },	/* ??? */
	{ SYS_issetugid,	"issetugid", 0 },
	{ SYS_lchown,	"lchown", 3, { STRING, INT, INT, } },
	{ SYS_getdents,	"getdents", 3, { INT, HEX, INT, } },
	{ SYS_lchmod,	"lchmod", 2, { STRING, INT, } },
	{ SYS_netbsd_lchown,	"netbsd_lchown", 1, { HEX, } },	/* ??? */
	{ SYS_lutimes,	"lutimes", 2, { STRING, TIMEVAL, } },
	{ SYS_netbsd_msync,	"netbsd_msync", 1, { HEX, } },	/* ??? */
	{ SYS_nstat,	"nstat", 1, { HEX, } },			/* ??? */
	{ SYS_nfstat,	"nfstat", 1, { HEX, } },		/* ??? */
	{ SYS_nlstat,	"nlstat", 1, { HEX, } },		/* ??? */
	{ SYS_fhstatfs,	"fhstatfs", 2, { INT, STATFS, } },
	{ SYS_fhopen,	"fhopen", 2, { HEX, INT, } },
	{ SYS_fhstat,	"fhstat", 2, { HEX, STAT, } },
	{ SYS_modnext,	"modnext", 1, { INT, } },
	{ SYS_modstat,	"modstat", 2, { INT, MODSTAT, } },
	{ SYS_modfnext,	"modfnext", 1, { INT, } },
	{ SYS_modfind,	"modfind", 1, { STRING, } },
	{ SYS_kldload,	"kldload", 1, { STRING, } },
	{ SYS_kldunload,	"kldunload", 1, { INT, } },
	{ SYS_kldfind,	"kldfind", 1, { STRING, } },
	{ SYS_kldnext,	"kldnext", 1, { INT, } },
	{ SYS_kldstat,	"kldstat", 2, { INT, KLDSTAT, } },
	{ SYS_kldfirstmod,	"kldfirstmod", 1, { INT, } },
	{ SYS_getsid,	"getsid", 1, { INT, } },
	{ SYS_setresuid,	"setresuid", 3, { INT, INT, INT, } },
	{ SYS_setresgid,	"setresgid", 3, { INT, INT, INT, } },
	{ SYS_aio_return,	"aio_return", 1, { HEX, } },
	{ SYS_aio_suspend,	"aio_suspend", 3, { HEX, INT, TIMESPEC, } },
	{ SYS_aio_cancel,	"aio_cancel", 2, { INT, HEX, } },
	{ SYS_aio_error,	"aio_error", 1, { HEX, } },
	{ SYS_aio_read,	"aio_read", 1, { HEX, } },
	{ SYS_aio_write,	"aio_write", 1, { HEX, } },
	{ SYS_lio_listio,	"lio_listio", 1, { HEX, } },	/* ??? */
	{ SYS_yield,	"yield", 1, { HEX, } },			/* ??? */
	{ SYS_thr_sleep,	"thr_sleep", 1, { HEX, } },	/* ??? */
	{ SYS_thr_wakeup,	"thr_wakeup", 1, { HEX, } },	/* ??? */
	{ SYS_mlockall,	"mlockall", 1, { HEX, } },		/* ??? */
	{ SYS_munlockall,	"munlockall", 1, { HEX, } },	/* ??? */
	{ SYS___getcwd,	"__getcwd", 1, { HEX, } },
	{ SYS_sched_setparam,	"sched_setparam", 2, { INT, SHEDPARAM, } },
	{ SYS_sched_getparam,	"sched_getparam", 2, { INT, SHEDPARAM, } },
	{ SYS_sched_setscheduler,	"sched_setscheduler", 3,
		{ INT, INT, SHEDPARAM, } },
	{ SYS_sched_getscheduler,	"sched_getscheduler", 1, { INT, } },
	{ SYS_sched_yield,	"sched_yield", 0 },
	{ SYS_sched_get_priority_max,	"sched_get_priority_max", 1, { INT, } },
	{ SYS_sched_get_priority_min,	"sched_get_priority_min", 1, { INT, } },
	{ SYS_sched_rr_get_interval,	"sched_rr_get_interval", 2,
		{ INT, TIMESPEC, } },
	{ SYS_utrace,	"utrace", 1, { HEX, } },		/* ??? */
	{ SYS_sendfile,	"sendfile", 7, { INT, INT, INT, INT, HEX, HEX, INT, } },
	{ SYS_kldsym,	"kldsym", 3, { INT, INT, HEX, } },
	{ SYS_jail,	"jail", 1, { HEX, } },
	{ SYS_sigprocmask,	"sigprocmask", 3, { INT, SIGSET, SIGSET, } },
	{ SYS_sigsuspend,	"sigsuspend", 1, { SIGSET, } },
	{ SYS_sigaction,	"sigaction", 3, { INT, SIGACTION, SIGACTION,} },
	{ SYS_sigpending,	"sigpending", 1, { SIGSET, } },
	{ SYS_sigreturn,	"sigreturn", 1, { HEX, } },
	{ SYS___acl_get_file,	"__acl_get_file", 2, { STRING, INT, } },
	{ SYS___acl_set_file,	"__acl_set_file", 3, { STRING, INT, INT, } },
	{ SYS___acl_get_fd,	"__acl_get_fd", 1, { INT, } },
	{ SYS___acl_set_fd,	"__acl_set_fd", 2, { INT, INT, } },
	{ SYS___acl_delete_file,	"__acl_delete_file", 1, { HEX, } },/* ??? */
	{ SYS___acl_delete_fd,	"__acl_delete_fd", 1, { HEX, } },/* ??? */
	{ SYS___acl_aclcheck_file,	"__acl_aclcheck_file", 1, { HEX, } },/* ??? */
	{ SYS___acl_aclcheck_fd,	"__acl_aclcheck_fd", 1, { HEX, } },	/* ??? */
	{ SYS_extattrctl,	"extattrctl", 1, { HEX, } },	/* ??? */
	{ SYS_extattr_set_file,	"extattr_set_file", 1, { HEX, } },/* ??? */
	{ SYS_extattr_get_file,	"extattr_get_file", 1, { HEX, } },/* ??? */
	{ SYS_extattr_delete_file,	"extattr_delete_file", 1, { HEX, } }, /*??? */
	{ SYS_aio_waitcomplete,	"aio_waitcomplete", 2, { HEX, TIMESPEC, } },
	{ SYS_getresuid,	"getresuid", 1, { HEX, } },	/* ??? */
	{ SYS_getresgid,	"getresgid", 1, { HEX, } },	/* ??? */
	{ SYS_kqueue,	"kqueue", 0 },
	{ SYS_kevent,	"kevent", 6,
		{ INT, KEVENT, INT, KEVENT, INT, TIMESPEC, } },
};

/*
 * Return syscall's name by the given number
 * Parameters:
 *   no - syscall number
 * Return:
 *   syscall's name
 */
const struct syscall *
xtr_lookup_syscall(int no)
{
	const struct syscall *retval = 0;
	int i;

	for (i = 0; i < sizeof(__syscalls)/sizeof(struct syscall); i++)
		if (__syscalls[i].no == no) {
			retval = &__syscalls[i];
			break;
		}

	return retval;
}

